import os
import sys
sys.path.append(os.path.abspath(os.path.dirname(__file__) + '/' + '../..'))

from itertools import product

import numpy as np
import pandas as pd

from python.tools import (
    clean_folder,
    get_inflation_forecasts
)

################
## Parameters ##
################

file_name_ind_forecasts = './construct_dataset/input/Individual_PGDP.xlsx'
file_name_actuals = './construct_dataset/input/p_first_second_third.xlsx'
output_folder = './construct_dataset/output'

#######################
## Construct dataset ##
#######################

clean_folder(output_folder)

df_ind = pd.read_excel(file_name_ind_forecasts)
df_actuals = pd.read_excel(file_name_actuals, sheet_name = 'DATA', skiprows = 4)
df = df_ind.copy()

# Create date variable
df['DATE'] = pd.to_datetime(df['YEAR'].map(str) + 'Q' + df['QUARTER'].map(str))

# Construct inflation forecasts
df = get_inflation_forecasts(df, melt_id_vars = ['DATE', 'ID'])

# Add back industry 
df_ind['DATE'] = df_ind['YEAR'].map(str) + ':0' + df_ind['QUARTER'].map(str)
df = pd.merge(df, df_ind.loc[:, ['DATE', 'ID', 'INDUSTRY']], 
              left_on = ['DATE', 'ID'], right_on = ['DATE', 'ID'], how = 'left')

# Data for 1995:Q4 is missing for first release, use second release
df_actuals.loc[df_actuals['Date'] == '1995:Q4', 'First'] = df_actuals.loc[df_actuals['Date'] == '1995:Q4', 'Second']
df_actuals.rename(columns = {'Date': 'DATE',
                             'First': 'Realiz1'}, inplace = True)
df_actuals['DATE'] = df_actuals['DATE'].map(lambda x: x.replace('Q', '0'))

# Merge forecasts and actuals
df = pd.merge(df, df_actuals.loc[:, ['DATE', 'Realiz1']], on = 'DATE', how = 'left')

# Export balanced panel, filling missing dates with NA
unique_ID = list(df['ID'].unique())
unique_dates = list(pd.Series(df['DATE'].unique()).sort_values().reset_index(drop = True))
df_balanced = pd.DataFrame(list(product(unique_ID, unique_dates)), columns = ['ID', 'DATE'])
df_balanced = df_balanced.merge(df, how = 'left', on = ['ID', 'DATE'])

# Save output to file
df_balanced.to_csv('{}/individual_error_statistics_PGDP.csv'.format(output_folder), index = False)