import datetime
import os
import sys
sys.path.append(os.path.abspath(os.path.dirname(__file__) + '/' + '../..'))

import matplotlib as mpl

mpl.use('pgf')
pgf_with_latex = {                     
    "pgf.texsystem": "xelatex",        
    "pgf.rcfonts": False,
    "text.usetex": True,                
    "font.family": "Times New Roman",
    "pgf.preamble": [
        r"\usepackage{fontspec}",    
        r"\setmainfont{Times New Roman}",        
        r"\usepackage{unicode-math}",
        r"\setmathfont{XITS Math}"
        ]
    }    
mpl.rcParams.update(pgf_with_latex)

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import statsmodels.api as sm
import statsmodels.formula.api as smf

from python.tools import (
    clean_folder
) 

################
## Parameters ##
################

input_folder = './graphs/input'
output_folder = './graphs/output/graph_factors'
dpi = 600

clean_folder(output_folder)

###########
## Graph ##
###########

# For correctly plotting dates later on
df_full = pd.read_csv('{}/individual_error_statistics_PGDP.csv'.format(input_folder))
df_full['DATE'] = pd.to_datetime(df_full['DATE'].apply(lambda x: x.replace(':0', 'Q')))
df_dates = pd.Series(df_full['DATE'].unique()).sort_values()
df_dates = pd.DataFrame(df_dates)
df_dates.columns = ['DATE']

# Graphing styles
var_names_list = ['SPFfor_Step{}'.format(ii) for ii in range(1, 5 + 1)]
line_styles = ['-.k', '--b', ':r', '-.g', '--m']

# Loop across forecasting horizons
for var_name, line_style in zip(var_names_list, line_styles):
    df = pd.read_csv('{}/baseline/df_resid_PCA_{}_number_factors_1.csv'.format(input_folder,
                                                                               var_name))
    df['DATE'] = pd.to_datetime(df['DATE'].apply(lambda x: x.replace(':0', 'Q')))
    df_cons = df.groupby('DATE').mean()
    df_cons = df_cons.reset_index()
    
    # Rotate factors to have the same scale
    # as realized inflation
    mod = smf.ols('Realiz1 ~ factor_1', data = df_cons).fit()
    df_cons['factor_1_rotated'] = mod.fittedvalues
    
    # Merge with full set of dates to properly plot
    df_temp = df_dates.copy()
    df_temp = pd.merge(df_temp, df_cons[['DATE', 'factor_1_rotated', 'Realiz1']], how = 'left', on = 'DATE')
    
    if var_name == 'SPFfor_Step1':
        fig, ax = plt.subplots(figsize=(4.5, 3.0))
        plt.plot(df_temp['DATE'], df_temp['Realiz1'], '-k', label = 'Inflation', alpha = 0.85)
    
    plt.plot(df_temp['DATE'], df_temp['factor_1_rotated'], 
             line_style, 
             label = '$h$ = {}, corr. = {:.2f}'.format(int(var_name[-1]) - 1, np.sqrt(mod.rsquared)), 
             linewidth = 1.0)
plt.legend(frameon = False, fontsize = 12)
plt.ylabel('Inflation (Percent)', fontsize = 12)
plt.savefig("{}/factors_graph.png".format(output_folder), dpi = dpi)
plt.savefig("{}/factors_graph.pgf".format(output_folder), dpi = dpi)
plt.show()